"""Console module for outputting to consoleself.

This module defines a class for outputting to the console,
with the capabilities of the rich package.
"""

from rich.console import Console as RichConsole


class Console(RichConsole):
    """Class for providing rich console output with custom methods
    Args:
        RichConsole: Inherits from rich.console.Console
    """

    def primary(self, format: str) -> None:
        """Prints text in blue color. Should be used for primary text output.

        Args:
            format (str): Text to be printed
        """
        self.print(f"[blue]{format}[/blue]")

    def success(self, format: str):
        """Prints text in green. Should be used for success messages.

        Args:
            format (str): Text to be printed
        """
        self.print(f"[green]{format}[/green]")

    def error(self, format: str):
        """Prints text in red color. Should be used for error messages.

        Args:
            format (str): Text to be printed
        """
        self.print(f"[bold red]{format}[/bold red]")

    def prompt_list(self, question: str, options: list[str]) -> int:
        """Prints a list of options, and prompts the user for an answer.
        It will continue to prompt the user until a valid answer is inputted.

        Args:
            question (str): The prompt message.
            options (list[str]): The different options that should be displayed.

        Returns:
            int: Returns the index of the chosen option.
        """
        for i, option in enumerate(options):
            self.print(f"[{i}]: {option}")

        def print_invalid_input():
            return self.error("Invalid input")

        try:
            question_idx = int(self.input(question))
        except ValueError:
            print_invalid_input()
            return self.prompt_list(question, options)

        if question_idx >= len(options) or question_idx < 0:
            print_invalid_input()
            return self.prompt_list(question, options)

        return question_idx


console = Console()



from .option import Option


class SingleHostOption(Option):
    def __init__(self, single_host: bool = True):
        if single_host:
            super().__init__(option="R", value="span[hosts=1]")
        else:
            super().__init__()



import click

from dtuhpc.cli.cli_config import CLIConfig
from dtuhpc.commands import BKill


@click.command()
@click.option("--kill-all", "-a", default=False, is_flag=True)
@click.option("--done", "-d", default=False, is_flag=True)
@click.option("--list-signals", "-l", default=False, is_flag=True)
@click.option("--remove-without-waiting", "-r", default=False, is_flag=True)
@click.option("--application-profile", "-app", default=None, type=str)
@click.option("--kill-reason", "-K", default=None, type=str)
@click.option("--group-name", "-g", default=None, type=str)
@click.option("--job-name", "-J", default=None, type=str)
@click.option("--host-name", "-m", default=None, type=str)
@click.option("--queue", "-q", default=None, type=str)
@click.option("--signal-name", "-s", default=None, type=str)
@click.option("--status", "-S", default=None, type=str)
@click.option("--zero", "-z", default=None, type=str)
@click.argument("job_ids", nargs=-1)
@click.pass_obj
def bkill(
    config: CLIConfig,
    kill_all,
    done,
    list_signals,
    remove_without_waiting,
    application_profile,
    kill_reason,
    group_name,
    job_name,
    host_name,
    queue,
    signal_name,
    status,
    zero,
    job_ids,
):
    """Kill a job."""
    conn = config.connection()

    cmd = BKill(conn)
    cmd.run(
        *job_ids,
        kill_all=kill_all,
        as_done=done,
        list_signals=list_signals,
        remove_without_waiting=remove_without_waiting,
        application_profile_name=application_profile,
        kill_reason=kill_reason,
        group_name=group_name,
        job_name=job_name,
        host_name=host_name,
        queue=queue,
        signal_name=signal_name,
        status=status,
        kill_all_satisfying=zero,
    )
    conn.close()



from .command import Command


class Nodestat(Command):
    base_command = "nodestat"

    def run(  # noqa: C901
        self,
        *queues: list[str],
        cpu: bool = False,
        features: bool = False,
        gpu: bool = False,
        gpu_model: bool = False,
        jobs: bool = False,
        queue_jobs: bool = False,
        load_utilization: bool = False,
        visual_load_utilization: bool = False,
        memory: bool = False,
        reserved_slots: bool = False,
    ):
        args = []

        if cpu:
            args.append("-f")
        if features:
            args.append("-F")
        if gpu:
            args.append("-g")
        if gpu_model:
            args.append("-G")
        if jobs:
            args.append("-j")
        if queue_jobs:
            args.append("-J")
        if load_utilization:
            args.append("-l")
        if visual_load_utilization:
            args.append("-L")
        if memory:
            args.append("-m")
        if reserved_slots:
            args.append("-r")

        args += queues

        return super().run(*args)



from typing import Optional, Union


class Option:
    option: Optional[str]
    value: Optional[Union[str, int]]
    config_name: str

    def __init__(
        self, option: Optional[str] = None, value: Optional[Union[str, int]] = None
    ):
        self.option = option
        self.value = value

    def _get_option_value(self):
        return (
            '"' + str(self.value) + '"'
            if isinstance(self.value, str)
            else str(self.value)
        )

    def get_option_line(self) -> str:
        if self.option is None:
            return ""

        return (
            f"#BSUB -{self.option} {self._get_option_value()}"
            if self.value is not None
            else f"#BSUB -{self.option}"
        )



import base64
import json
import os

import click
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

from dtuhpc.cli.cli_config import CLIConfig
from dtuhpc.console import console


@click.command()
def auth():
    """Authenticate with the DTU HPC cluster."""
    auth_path = CLIConfig.get_global_auth_path()

    if auth_path.exists():
        click.confirm(
            f"Auth file already exists at {auth_path}. Do you want to overwrite it?",
            abort=True,
        )

    username = click.prompt("Username")
    ssh_password = bytes(click.prompt("Password", hide_input=True), "utf-8")

    console.primary(
        "You will now be asked to enter an encryption key, \
        for encrypting your password."
    )
    encryption_key = bytes(click.prompt("Encryption key", hide_input=True), "utf-8")

    salt = os.urandom(16)
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=480000,
    )
    key = base64.urlsafe_b64encode(kdf.derive(encryption_key))

    f = Fernet(key)

    auth = {
        "username": username,
        "password": f.encrypt(ssh_password).decode("utf-8"),
        "salt": base64.urlsafe_b64encode(salt).decode("utf-8"),
    }
    auth_path.parent.mkdir(parents=True, exist_ok=True)

    with open(auth_path, "w") as auth_file:
        json.dump(auth, auth_file)

    console.success(f"Successfully saved auth file to {auth_path}")



import click

from dtuhpc.cli.cli_config import CLIConfig
from dtuhpc.cli.commands.server.bkill import bkill
from dtuhpc.cli.commands.server.bqueues import bqueues
from dtuhpc.cli.commands.server.bstat import bstat
from dtuhpc.cli.commands.server.bsub import bsub
from dtuhpc.cli.commands.server.nodestat import nodestat
from dtuhpc.cli.commands.server.showstart import showstart


@click.group(name="c")
@click.pass_obj
def server_command(config: CLIConfig):
    config.load_config()
    """Execute a predefined command."""


server_command.add_command(bkill)
server_command.add_command(nodestat)
server_command.add_command(bqueues)
server_command.add_command(bstat)
server_command.add_command(bsub)
server_command.add_command(showstart)



import click

from dtuhpc.cli.cli_config import CLIConfig
from dtuhpc.commands import Nodestat


@click.command()
@click.option("--cpu", "-c", default=False, is_flag=True)
@click.option("--features", "-f", default=False, is_flag=True)
@click.option("--gpu", "-g", default=False, is_flag=True)
@click.option("--gpu_model", "-G", default=False, is_flag=True)
@click.option("--jobs", "-j", default=False, is_flag=True)
@click.option("--queue_jobs", "-J", default=False, is_flag=True)
@click.option("--load_util", "-l", default=False, is_flag=True)
@click.option("--visual_load_util", "-v", default=False, is_flag=True)
@click.option("--memory", "-m", default=False, is_flag=True)
@click.option("--reserved", "-r", default=False, is_flag=True)
@click.argument("queues", nargs=-1)
@click.pass_obj
def nodestat(
    config: CLIConfig,
    cpu,
    features,
    gpu,
    gpu_model,
    jobs,
    queue_jobs,
    load_util,
    visual_load_util,
    memory,
    reserved,
    queues,
):
    """Print the nodestat from the server."""
    conn = config.connection()

    cmd = Nodestat(conn)
    cmd.run(
        *queues,
        cpu=cpu,
        features=features,
        gpu=gpu,
        gpu_model=gpu_model,
        jobs=jobs,
        queue_jobs=queue_jobs,
        load_utilization=load_util,
        visual_load_utilization=visual_load_util,
        memory=memory,
        reserved_slots=reserved,
    )

    conn.close()



from .option import Option


class ErrorOutputFilePathOption(Option):
    name: str
    suffix_job_id: bool
    overwrite: bool

    def __init__(self, name: str, suffix_job_id: bool = True, overwrite: bool = False):
        self.name = name
        self.suffix_job_id = suffix_job_id
        self.overwrite = overwrite

        super().__init__(option=self.get_option(), value=self.get_job_name())

    def get_option(self):
        return "e" if not self.overwrite else "eo"

    def get_job_name(self):
        return self.name if self.suffix_job_id else f"{self.name}_%J"



import click

from dtuhpc.cli.cli_config import CLIConfig
from dtuhpc.commands import BSub


@click.command()
@click.argument("script_path", nargs=1, type=str)
@click.pass_obj
def bsub(config: CLIConfig, script_path: str):
    """Submit a job to queue."""
    conn = config.connection()

    cmd = BSub(conn)
    cmd.run(f"< {script_path}")
    conn.close()



from typing import Union

from fabric.runners import Result

from dtuhpc.connector import HPCConnection


class Command:
    """Base class for all commands."""

    base_command: str
    connection: HPCConnection

    def __init__(self, connection: HPCConnection):
        self.connection = connection

    def run(self, *args: Union[str, int, float]) -> Result:
        parsed_args = " ".join([str(arg) for arg in args])

        command = self.base_command + " " + parsed_args
        cmd = self.connection.run(command)
        return cmd



from typing import Optional

from .command import Command


class BKill(Command):
    base_command = "bkill"

    def run(  # noqa: C901
        self,
        *job_ids: list[str],
        kill_all: bool = False,
        as_done: bool = False,
        list_signals: bool = False,
        remove_without_waiting: False = True,
        application_profile_name: Optional[str] = None,
        kill_reason: Optional[str] = None,
        group_name: Optional[str] = None,
        job_name: Optional[str] = None,
        host_name: Optional[str] = None,
        queue: Optional[str] = None,
        signal_name: Optional[str] = None,
        status: Optional[str] = None,
        kill_all_satisfying: bool = False,
    ):
        args = []

        if kill_all:
            args.append("-b")
        if as_done:
            args.append("-d")
        if list_signals:
            args.append("-l")
        if remove_without_waiting:
            args.append("-r")
        if application_profile_name is not None:
            args.append(f"-app {application_profile_name}")
        if kill_reason is not None:
            args.append(f"-C {kill_reason}")
        if group_name is not None:
            args.append(f"-g {group_name}")
        if job_name is not None:
            args.append(f"-J {job_name}")
        if host_name is not None:
            args.append(f"-m {host_name}")
        if queue is not None:
            args.append(f"-q {queue}")
        if signal_name is not None:
            args.append(f"-s {signal_name}")
        if status is not None:
            args.append(f"-stat {status}")
        if kill_all_satisfying:
            args.append("0")

        args += job_ids

        return super().run(*args)



from .bkill import BKill
from .bqueues import BQueues
from .bstat import BStat
from .bsub import BSub
from .command import Command
from .nodestat import Nodestat
from .showstart import Showstart

__all__ = ["BKill", "BQueues", "BStat", "BSub", "Showstart", "Nodestat", "Command"]



"""Connection module.

This module defines the connection to the DTU HPC server.
"""
from typing import Optional

from fabric import Connection, Result


class HPCConnection:
    """Class for connecting and running commands on DTU HPC server.

    Attributes:
        _conn (Connection): Connection object from the fabric library.
        cwd (Optional[str]): Current working directory.
        hide (bool): Hide the output of the command.
    """

    conn: Connection
    cwd: Optional[str]
    hide: bool

    def __init__(
        self,
        user: str,
        host: str,
        password: str = None,
        hide: bool = False,
        cwd: Optional[str] = None,
        **kwargs,
    ):
        """Initialize the connection with the required parameters.

        Args:
            user (str): Username for the DTU HPC server.
            host (str): Hostname for the DTU HPC server.
            password (str, optional): Password for the DTU HPC server. Defaults to None.
            hide (bool, optional): Hide the output of the command. Defaults to False.
            cwd (Optional[str], optional): Current working directory. Defaults to None.
            **kwargs: Additional arguments parsed directly into Paramikos Client.
        """
        connect_kwargs = {"password": password} if password is not None else None
        self.hide = hide
        self.cwd = cwd
        self.conn = Connection(
            user=user, host=host, connect_kwargs=connect_kwargs, **kwargs
        )

    def run(self, command: str) -> Result:
        """Run a command on the DTU HPC server.

        Args:
            command (str): Command to run.

        Returns:
            Result: Result object from the fabric library.
        """

        if self.cwd is not None:
            with self.conn.cd(self.cwd):
                return self.conn.run(f"bash -l -c '{command}'", hide=self.hide)

        return self.conn.run(f"bash -l -c '{command}'", hide=self.hide)

    def open_shell(self) -> None:
        """Open a shell on the DTU HPC server."""
        self.conn.shell()

    def close(self):
        """Close the connection to the DTU HPC server."""
        self.conn.close()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dtuhpc',
 'dtuhpc.cli',
 'dtuhpc.cli.commands',
 'dtuhpc.cli.commands.server',
 'dtuhpc.commands',
 'dtuhpc.jobwriter',
 'dtuhpc.jobwriter.commands',
 'dtuhpc.jobwriter.options']

package_data = \
{'': ['*']}

install_requires = \
['cryptography>=39.0.1,<40.0.0',
 'fabric>=3.0.0,<4.0.0',
 'gitpython>=3.1.30,<4.0.0',
 'pygithub>=1.57,<2.0',
 'rich>=13.2.0,<14.0.0',
 'tomli>=2.0.1,<3.0.0']

entry_points = \
{'console_scripts': ['dtuhpc = dtuhpc.cli.cli:main']}

setup_kwargs = {
    'name': 'dtuhpc',
    'version': '1.0.4',
    'description': '',
    'long_description': '# DTU HPC',
    'author': 'Jonas Hoffmannn',
    'author_email': 's204071@student.dtu.dk',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
